#include <math.h>

#include "attenuation.h"

#include "liebe_line_database.h"

/* The Van Vleck-Weisskopf line shape function (modified by Rosenkranz
   1975). All input parameters are in GHz except overlap_correction.  */
static
afloat
line_shape(afloat frequency,
	   afloat line_centre,
	   afloat line_width,
	   afloat overlap_correction)
{
  afloat width2 = line_width * line_width;
  afloat n = line_centre - frequency;
  afloat m = line_centre + frequency;
  afloat x = 1.0 / (n * n + width2);
  afloat y = 1.0 / (m * m + width2);

  return  (frequency / line_centre)
    * ((x + y) * line_width - ( n*x + m*y ) * overlap_correction);
}

afloat
liebe_attenuation(afloat frequency_GHz,
		  afloat temperature_K,
		  afloat pressure_Pa,
		  afloat vapour_pressure_Pa)

{
  /* theta is the "Relative Inverse Temperature" */
  afloat theta = 300.0 / temperature_K;
  afloat pressure_kPa = pressure_Pa * 0.001;
  afloat vapour_pressure_kPa = vapour_pressure_Pa * 0.001;
  
  /* Complex part of the refractivity from different sources */
  afloat N_O2_lines     = 0.0;  /* Oxygen lines */
  afloat N_H2O_lines    = 0.0;  /* Water vapour lines */
  afloat N_dry_air_cont = 0.0;  /* Dry air continuum */
  afloat N_H2O_cont1    = 0.0;  /* Water vapour continuum 1 */
  afloat N_H2O_cont2    = 0.0;  /* Water vapour continuum 2 */

  afloat gamma0; /* Dry air continuum parameter */
  
  /* Optimisation saves */
  afloat pressure_theta3 = pressure_kPa * theta * theta * theta;
  afloat theta_0p8 = pow(theta, 0.8);
  afloat theta_2p5 = pow(theta, 2.5);
  afloat frequency_1p5 = pow(frequency_GHz, 1.5);

  int k;

  /* Calculate N due to oxygen lines */
  for (k = 0; k < LIEBE_NUM_O2_LINES; k++)
    {
      N_O2_lines += liebe_O2_c(1, k) * pressure_theta3
	* exp(liebe_O2_c(2, k) * (1.0-theta))
	* line_shape(frequency_GHz, liebe_O2_f(k), liebe_O2_c(3, k)
		       * (pressure_kPa * pow(theta, 0.8-liebe_O2_c(4, k))
			  + 1.1 * vapour_pressure_kPa * theta),
		     liebe_O2_c(5, k) * pressure_kPa
		       * pow(theta, liebe_O2_c(6, k)));
    }

  /* Calculate N due to water vapour lines */
  for (k = 0; k < LIEBE_NUM_H2O_LINES; k++)
    {
      N_H2O_lines += liebe_H2O_c(1, k) * vapour_pressure_kPa * pow(theta, 3.5)
	* exp(liebe_H2O_c(2, k) * (1.0-theta))
	* line_shape(frequency_GHz, liebe_H2O_f(k), liebe_H2O_c(3, k)
		       * (pressure_kPa * theta_0p8 
			  + 4.8 * vapour_pressure_kPa * theta),
		     0.0);
    }

  /* Calculate N due to the dry air continuum */
  gamma0 = 5.6e-3 * (pressure_kPa + 1.1 * vapour_pressure_kPa) * theta_0p8;
  N_dry_air_cont = (2.0 * LIEBE_DRY_AIR_CONT_COEFFT
		    / (gamma0 * (1.0 + (frequency_GHz/60.0)
				     * (frequency_GHz/60.0))
		       * (1.0 + (frequency_GHz/gamma0)
			      * (frequency_GHz/gamma0)))
		    + 1.4e-10 * (1.0 - 1.2e-5 * frequency_1p5)
		    * pressure_kPa * theta_2p5)
    * frequency_GHz * pressure_kPa * theta * theta;

  /* Calculate N due to the water vapour continuum.  An "absorption
     correction" is included (Liebe 1985) to account for the
     difference between this reduced line base model with the full
     line model. */
  N_H2O_cont1 = (LIEBE_H2O_CONT_COEFFT1 * pressure_kPa
		 + LIEBE_H2O_CONT_COEFFT2 * vapour_pressure_kPa
		   * theta * theta * theta)
    * frequency_GHz * vapour_pressure_kPa * theta_2p5;

  N_H2O_cont2 = 2.9e-8 * pressure_kPa * pow(vapour_pressure_kPa, 1.1)
    * theta * theta * frequency_1p5;

  return 0.182 * frequency_GHz
    * (N_O2_lines + N_H2O_lines + N_dry_air_cont + N_H2O_cont1 + N_H2O_cont2);

}

void
liebe_attenuation_profile(afloat *height_m,
			  afloat *pressure_Pa,
			  afloat *temperature_K,
			  afloat *vapour_pressure_Pa,
			  int nlevels,
			  afloat frequency_GHz,
			  /* out */ afloat *attenuation_dB,
			  afloat *attenuation_dB_km)
{
  afloat dB_km;
  afloat last_dB_km = liebe_attenuation(frequency_GHz,
					temperature_K[0],
					pressure_Pa[0],
					vapour_pressure_Pa[0]);
  int n;

  attenuation_dB[0] = 0.0;
  if (attenuation_dB_km) {
    *attenuation_dB_km = last_dB_km;
  }

  for (n = 1; n < nlevels; n++) {
    dB_km = liebe_attenuation(frequency_GHz,
			      temperature_K[n],
			      pressure_Pa[n],
			      vapour_pressure_Pa[n]);
    /* Note: divide by 1000 to get from m to km, multiply by 2 to get
       two-way value, divide by two because we're averaging the
       specific attenuation at this gate (dB_km) and the last gate
       (last_dB_km) */
    attenuation_dB[n] = attenuation_dB[n-1]
      + (last_dB_km + dB_km)
      * (height_m[n] - height_m[n-1]) * 0.001;

    if (attenuation_dB_km) {
      attenuation_dB_km[n] = dB_km;
    }
    last_dB_km = dB_km;
  }
}
